@@ -1,22 +1,12 @@
 #!/usr/bin/python
 # Copyright 2016 Google Inc.
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -206,7 +196,7 @@ named_ports:
 size:
     description: Number of VMs in Managed Instance Group.
     returned: changed
-    type: integer
+    type: int
     sample: 4
 
 created_instances:
@@ -272,7 +262,11 @@ updated_named_ports:
     sample: true
 '''
 
-import socket
+try:
+    from ast import literal_eval
+    HAS_PYTHON26 = True
+except ImportError:
+    HAS_PYTHON26 = False
 
 try:
     import libcloud
@@ -286,11 +280,8 @@ try:
 except ImportError:
     HAS_LIBCLOUD = False
 
-try:
-    from ast import literal_eval
-    HAS_PYTHON26 = True
-except ImportError:
-    HAS_PYTHON26 = False
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.gce import gce_connect
 
 
 def _check_params(params, field_list):
@@ -348,7 +339,7 @@ def _validate_autoscaling_params(params):
         {'name': 'name', 'required': True, 'type': str},
         {'name': 'enabled', 'required': True, 'type': bool},
         {'name': 'policy', 'required': True, 'type': dict}
-    ] # yapf: disable
+    ]  # yapf: disable
 
     (as_req_valid, as_req_msg) = _check_params(params['autoscaling'],
                                                as_req_fields)
@@ -360,7 +351,7 @@ def _validate_autoscaling_params(params):
         {'name': 'max_instances', 'required': True, 'type': int},
         {'name': 'min_instances', 'required': False, 'type': int},
         {'name': 'cool_down_period', 'required': False, 'type': int}
-    ] # yapf: disable
+    ]  # yapf: disable
 
     (as_policy_valid, as_policy_msg) = _check_params(
         params['autoscaling']['policy'], as_policy_fields)
@@ -394,7 +385,7 @@ def _validate_named_port_params(params):
     req_fields = [
         {'name': 'name', 'required': True, 'type': str},
         {'name': 'port', 'required': True, 'type': int}
-    ] # yapf: disable
+    ]  # yapf: disable
 
     for np in params['named_ports']:
         (valid_named_ports, np_msg) = _check_params(np, req_fields)
@@ -404,7 +395,7 @@ def _validate_named_port_params(params):
     return (True, '')
 
 
-def _get_instance_list(mig, field='name', filter_list=['NONE']):
+def _get_instance_list(mig, field='name', filter_list=None):
     """
     Helper to grab field from instances response.
 
@@ -423,6 +414,8 @@ def _get_instance_list(mig, field='name', filter_list=['NONE']):
     :return: List of strings from list_managed_instances response.
     :rtype: ``list``
     """
+    filter_list = ['NONE'] if filter_list is None else filter_list
+
     return [x[field] for x in mig.list_managed_instances()
             if x['currentAction'] in filter_list]
 
@@ -779,7 +772,7 @@ def main():
             req_create_fields = [
                 {'name': 'template', 'required': True, 'type': str},
                 {'name': 'size', 'required': True, 'type': int}
-            ] # yapf: disable
+            ]  # yapf: disable
 
             (valid_create_fields, valid_create_msg) = _check_params(
                 params, req_create_fields)
@@ -902,8 +895,6 @@ def main():
     json_output.update(params)
     module.exit_json(**json_output)
 
-# import module snippets
-from ansible.module_utils.basic import *
-from ansible.module_utils.gce import *
+
 if __name__ == '__main__':
     main()
