@@ -377,7 +377,7 @@ EXAMPLES = """
     - test-container-new-archive-destroyed-clone
 """
 
-RETURN="""
+RETURN = """
 lxc_container:
     description: container information
     returned: success
@@ -579,7 +579,7 @@ def create_script(command):
         f.close()
 
     # Ensure the script is executable.
-    os.chmod(script_file, int('0700',8))
+    os.chmod(script_file, int('0700', 8))
 
     # Output log file.
     stdout_file = os.fdopen(tempfile.mkstemp(prefix='lxc-attach-script-log')[0], 'ab')
@@ -669,8 +669,7 @@ class LxcContainerManagement(object):
             build_command.append(
                 '%s %s' % (key, value)
             )
-        else:
-            return build_command
+        return build_command
 
     def _get_vars(self, variables):
         """Return a dict of all variables as found within the module.
@@ -690,8 +689,7 @@ class LxcContainerManagement(object):
             _var = self.module.params.get(k)
             if _var not in false_values:
                 return_dict[v] = _var
-        else:
-            return return_dict
+        return return_dict
 
     def _run_command(self, build_command, unsafe_shell=False):
         """Return information from running an Ansible Command.
@@ -917,7 +915,7 @@ class LxcContainerManagement(object):
             'ips': self.container.get_ips(),
             'state': self._get_state(),
             'init_pid': int(self.container.init_pid),
-            'name' : self.container_name,
+            'name': self.container_name,
         }
 
     def _unfreeze(self):
@@ -976,16 +974,15 @@ class LxcContainerManagement(object):
                 time.sleep(1)
             else:
                 return True
-        else:
-            self.failure(
-                lxc_container=self._container_data(),
-                error='Failed to start container'
-                      ' [ %s ]' % self.container_name,
-                rc=1,
-                msg='The container [ %s ] failed to start. Check to lxc is'
-                    ' available and that the container is in a functional'
-                    ' state.' % self.container_name
-            )
+        self.failure(
+            lxc_container=self._container_data(),
+            error='Failed to start container'
+                  ' [ %s ]' % self.container_name,
+            rc=1,
+            msg='The container [ %s ] failed to start. Check to lxc is'
+                ' available and that the container is in a functional'
+                ' state.' % self.container_name
+        )
 
     def _check_archive(self):
         """Create a compressed archive of a container.
@@ -1368,7 +1365,7 @@ class LxcContainerManagement(object):
         :type source_dir: ``str``
         """
 
-        old_umask = os.umask(int('0077',8))
+        old_umask = os.umask(int('0077', 8))
 
         archive_path = self.module.params.get('archive_path')
         if not os.path.isdir(archive_path):
@@ -1753,7 +1750,7 @@ def main():
             )
         ),
         supports_check_mode=False,
-        required_if = ([
+        required_if=([
             ('archive', True, ['archive_path'])
         ]),
     )
