@@ -268,9 +268,6 @@ class HostsModule(BaseModule):
             ssh=otypes.Ssh(
                 authentication_method=otypes.SshAuthenticationMethod.PUBLICKEY,
             ) if self.param('public_key') else None,
-            kdump_status=otypes.KdumpStatus(
-                self.param('kdump_integration')
-            ) if self.param('kdump_integration') else None,
             spm=otypes.Spm(
                 priority=self.param('spm_priority'),
             ) if self.param('spm_priority') else None,
@@ -283,14 +280,15 @@ class HostsModule(BaseModule):
             ) if self.param('kernel_params') else None,
             power_management=otypes.PowerManagement(
                 enabled=self.param('power_management_enabled'),
-            ) if self.param('power_management_enabled') is not None else None,
+                kdump_detection=self.param('kdump_integration') == 'enabled',
+            ) if self.param('power_management_enabled') is not None or self.param('kdump_integration') else None,
         )
 
     def update_check(self, entity):
         kernel_params = self.param('kernel_params')
         return (
             equal(self.param('comment'), entity.comment) and
-            equal(self.param('kdump_integration'), entity.kdump_status) and
+            equal(self.param('kdump_integration'), 'enabled' if entity.power_management.kdump_detection else 'disabled') and
             equal(self.param('spm_priority'), entity.spm.priority) and
             equal(self.param('power_management_enabled'), entity.power_management.enabled) and
             equal(self.param('override_display'), getattr(entity.display, 'address', None)) and
@@ -476,8 +474,11 @@ def main():
                             event
                             for event in events_service.list(
                                 from_=int(last_event.id),
-                                search='type=885 and host.name=%s' % host.name,
-                            )
+                                search='type=885',
+                                # Uncomment when 4.1 is EOL, and remove the cond:
+                                # if host.name in event.description
+                                # search='type=885 and host.name=%s' % host.name,
+                            ) if host.name in event.description
                         ]) > 0
                     ),
                     fail_condition=lambda host: len([
@@ -499,12 +500,13 @@ def main():
             )
         elif state == 'iscsidiscover':
             host_id = get_id_by_name(hosts_service, module.params['name'])
+            iscsi_param = module.params['iscsi']
             iscsi_targets = hosts_service.service(host_id).iscsi_discover(
                 iscsi=otypes.IscsiDetails(
-                    port=int(module.params['iscsi']['port']) if module.params['iscsi']['port'].isdigit() else None,
-                    username=module.params['iscsi']['username'],
-                    password=module.params['iscsi']['password'],
-                    address=module.params['iscsi']['address'],
+                    port=int(iscsi_param.get('port', 3260)),
+                    username=iscsi_param.get('username'),
+                    password=iscsi_param.get('password'),
+                    address=iscsi_param.get('address'),
                 ),
             )
             ret = {
@@ -514,14 +516,15 @@ def main():
             }
         elif state == 'iscsilogin':
             host_id = get_id_by_name(hosts_service, module.params['name'])
+            iscsi_param = module.params['iscsi']
             ret = hosts_module.action(
                 action='iscsi_login',
                 iscsi=otypes.IscsiDetails(
-                    port=int(module.params['iscsi']['port']) if module.params['iscsi']['port'].isdigit() else None,
-                    username=module.params['iscsi']['username'],
-                    password=module.params['iscsi']['password'],
-                    address=module.params['iscsi']['address'],
-                    target=module.params['iscsi']['target'],
+                    port=int(iscsi_param.get('port', 3260)),
+                    username=iscsi_param.get('username'),
+                    password=iscsi_param.get('password'),
+                    address=iscsi_param.get('address'),
+                    target=iscsi_param.get('target'),
                 ),
             )
         elif state == 'started':
