@@ -1,46 +1,37 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017 F5 Networks Inc.
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2017 F5 Networks Inc.
+# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
-
-DOCUMENTATION = '''
+DOCUMENTATION = r'''
 ---
 module: bigip_gtm_wide_ip
-short_description: Manages F5 BIG-IP GTM wide ip.
+short_description: Manages F5 BIG-IP GTM wide ip
 description:
   - Manages F5 BIG-IP GTM wide ip.
 version_added: "2.0"
 options:
-  lb_method:
+  pool_lb_method:
     description:
       - Specifies the load balancing method used to select a pool in this wide
         IP. This setting is relevant only when multiple pools are configured
         for a wide IP.
     required: True
+    aliases: ['lb_method']
     choices:
       - round-robin
       - ratio
       - topology
       - global-availability
+    version_added: 2.5
   name:
     description:
       - Wide IP name. This name must be formatted as a fully qualified
@@ -66,9 +57,9 @@ options:
   state:
     description:
       - When C(present) or C(enabled), ensures that the Wide IP exists and
-        is enabled. When C(absent), ensures that the Wide IP has been
-        removed. When C(disabled), ensures that the Wide IP exists and is
-        disabled.
+        is enabled.
+      - When C(absent), ensures that the Wide IP has been removed.
+      - When C(disabled), ensures that the Wide IP exists and is disabled.
     default: present
     choices:
       - present
@@ -76,76 +67,143 @@ options:
       - disabled
       - enabled
     version_added: 2.4
-notes:
-  - Requires the f5-sdk Python package on the host. This is as easy as pip
-    install f5-sdk.
+  partition:
+    description:
+      - Device partition to manage resources on.
+    default: Common
+    version_added: 2.5
+  pools:
+    description:
+      - The pools that you want associated with the Wide IP.
+      - If C(ratio) is not provided when creating a new Wide IP, it will default
+        to 1.
+    suboptions:
+      name:
+        description:
+          - The name of the pool to include
+        required: true
+      ratio:
+        description:
+          - Ratio for the pool.
+          - The system uses this number with the Ratio load balancing method.
+    version_added: 2.5
 extends_documentation_fragment: f5
-requirements:
-  - f5-sdk
 author:
   - Tim Rupp (@caphrim007)
 '''
 
-EXAMPLES = '''
+EXAMPLES = r'''
 - name: Set lb method
   bigip_gtm_wide_ip:
-      server: "lb.mydomain.com"
-      user: "admin"
-      password: "secret"
-      lb_method: "round-robin"
-      name: "my-wide-ip.example.com"
+    server: lb.mydomain.com
+    user: admin
+    password: secret
+    lb_method: round-robin
+    name: my-wide-ip.example.com
   delegate_to: localhost
 '''
 
-RETURN = '''
+RETURN = r'''
 lb_method:
-    description: The new load balancing method used by the wide IP.
-    returned: changed
-    type: string
-    sample: "topology"
+  description: The new load balancing method used by the wide IP.
+  returned: changed
+  type: string
+  sample: topology
 state:
-    description: The new state of the wide IP.
-    returned: changed
-    type: string
-    sample: "disabled"
+  description: The new state of the wide IP.
+  returned: changed
+  type: string
+  sample: disabled
 '''
 
 import re
 
-from ansible.module_utils.f5_utils import (
-    AnsibleF5Client,
-    AnsibleF5Parameters,
-    HAS_F5SDK,
-    F5ModuleError,
-    iControlUnexpectedHTTPError
-)
+from ansible.module_utils.six import iteritems
 from distutils.version import LooseVersion
 
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.basic import env_fallback
+
+HAS_DEVEL_IMPORTS = False
+
+try:
+    # Sideband repository used for dev
+    from library.module_utils.network.f5.bigip import HAS_F5SDK
+    from library.module_utils.network.f5.bigip import F5Client
+    from library.module_utils.network.f5.common import F5ModuleError
+    from library.module_utils.network.f5.common import AnsibleF5Parameters
+    from library.module_utils.network.f5.common import cleanup_tokens
+    from library.module_utils.network.f5.common import fqdn_name
+    from library.module_utils.network.f5.common import f5_argument_spec
+    try:
+        from library.module_utils.network.f5.common import iControlUnexpectedHTTPError
+    except ImportError:
+        HAS_F5SDK = False
+    HAS_DEVEL_IMPORTS = True
+except ImportError:
+    # Upstream Ansible
+    from ansible.module_utils.network.f5.bigip import HAS_F5SDK
+    from ansible.module_utils.network.f5.bigip import F5Client
+    from ansible.module_utils.network.f5.common import F5ModuleError
+    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
+    from ansible.module_utils.network.f5.common import cleanup_tokens
+    from ansible.module_utils.network.f5.common import fqdn_name
+    from ansible.module_utils.network.f5.common import f5_argument_spec
+    try:
+        from ansible.module_utils.network.f5.common import iControlUnexpectedHTTPError
+    except ImportError:
+        HAS_F5SDK = False
+
 
 class Parameters(AnsibleF5Parameters):
-    updatables = ['lb_method']
-    returnables = ['name', 'lb_method', 'state']
-    api_attributes = ['poolLbMode', 'enabled', 'disabled']
+    api_map = {
+        'poolLbMode': 'pool_lb_method'
+    }
+    updatables = ['pool_lb_method', 'state', 'pools']
+    returnables = ['name', 'pool_lb_method', 'state', 'pools']
+    api_attributes = ['poolLbMode', 'enabled', 'disabled', 'pools']
 
-    def to_return(self):
-        result = {}
-        for returnable in self.returnables:
-            result[returnable] = getattr(self, returnable)
-        result = self._filter_params(result)
-        return result
+    def _fqdn_name(self, value):
+        if value is not None and not value.startswith('/'):
+            return '/{0}/{1}'.format(self.partition, value)
+        return value
 
-    def api_params(self):
-        result = {}
-        for api_attribute in self.api_attributes:
-            if self.api_map is not None and api_attribute in self.api_map:
-                result[api_attribute] = getattr(self, self.api_map[api_attribute])
-            else:
-                result[api_attribute] = getattr(self, api_attribute)
-        result = self._filter_params(result)
-        return result
+
+class ApiParameters(Parameters):
+    @property
+    def disabled(self):
+        if self._values['disabled'] is True:
+            return True
+        return False
 
     @property
-    def lb_method(self):
+    def enabled(self):
+        if self._values['enabled'] is True:
+            return True
+        return False
+
+    @property
+    def pools(self):
+        result = []
+        if self._values['pools'] is None:
+            return None
+        pools = sorted(self._values['pools'], key=lambda x: x['order'])
+        for item in pools:
+            pool = dict()
+            pool.update(item)
+            name = '/{0}/{1}'.format(item['partition'], item['name'])
+            del pool['nameReference']
+            del pool['order']
+            del pool['name']
+            del pool['partition']
+            pool['name'] = name
+            result.append(pool)
+        return result
+
+
+class ModuleParameters(Parameters):
+    @property
+    def pool_lb_method(self):
         deprecated = [
             'return_to_dns', 'null', 'static_persist', 'vs_capacity',
             'least_conn', 'lowest_rtt', 'lowest_hops', 'packet_rate', 'cpu',
@@ -181,25 +239,6 @@ class Parameters(AnsibleF5Parameters):
             lb_method = 'round-robin'
         return lb_method
 
-    @lb_method.setter
-    def lb_method(self, value):
-        self._values['lb_method'] = value
-
-    @property
-    def collection(self):
-        type_map = dict(
-            a='a_s',
-            aaaa='aaaas',
-            cname='cnames',
-            mx='mxs',
-            naptr='naptrs',
-            srv='srvs'
-        )
-        if self._values['type'] is None:
-            return None
-        wideip_type = self._values['type']
-        return type_map[wideip_type]
-
     @property
     def type(self):
         if self._values['type'] is None:
@@ -216,14 +255,6 @@ class Parameters(AnsibleF5Parameters):
             )
         return self._values['name']
 
-    @property
-    def poolLbMode(self):
-        return self.lb_method
-
-    @poolLbMode.setter
-    def poolLbMode(self, value):
-        self.lb_method = value
-
     @property
     def state(self):
         if self._values['state'] == 'enabled':
@@ -236,8 +267,6 @@ class Parameters(AnsibleF5Parameters):
             return False
         elif self._values['state'] in ['present', 'enabled']:
             return True
-        elif self._values['enabled'] is True:
-            return True
         else:
             return None
 
@@ -247,15 +276,111 @@ class Parameters(AnsibleF5Parameters):
             return True
         elif self._values['state'] in ['present', 'enabled']:
             return False
-        elif self._values['disabled'] is True:
-            return True
         else:
             return None
 
+    @property
+    def pools(self):
+        result = []
+        if self._values['pools'] is None:
+            return None
+        for item in self._values['pools']:
+            pool = dict()
+            if 'ratio' in item:
+                pool['ratio'] = item['ratio']
+            pool['name'] = self._fqdn_name(item['name'])
+            result.append(pool)
+        return result
+
+
+class Changes(Parameters):
+    def to_return(self):
+        result = {}
+        try:
+            for returnable in self.returnables:
+                change = getattr(self, returnable)
+                if isinstance(change, dict):
+                    result.update(change)
+                else:
+                    result[returnable] = change
+            result = self._filter_params(result)
+        except Exception:
+            pass
+        return result
+
+
+class UsableChanges(Changes):
+    pass
+
+
+class ReportableChanges(Changes):
+    @property
+    def pool_lb_method(self):
+        result = dict(
+            lb_method=self._values['pool_lb_method'],
+            pool_lb_method=self._values['pool_lb_method'],
+        )
+        return result
+
+
+class Difference(object):
+    def __init__(self, want, have=None):
+        self.want = want
+        self.have = have
+
+    def compare(self, param):
+        try:
+            result = getattr(self, param)
+            return result
+        except AttributeError:
+            return self.__default(param)
+
+    def __default(self, param):
+        attr1 = getattr(self.want, param)
+        try:
+            attr2 = getattr(self.have, param)
+            if attr1 != attr2:
+                return attr1
+        except AttributeError:
+            return attr1
+
+    def to_tuple(self, items):
+        result = []
+        for x in items:
+            tmp = [(str(k), str(v)) for k, v in iteritems(x)]
+            result += tmp
+        return result
+
+    def _diff_complex_items(self, want, have):
+        if want == [] and have is None:
+            return None
+        if want is None:
+            return None
+        w = self.to_tuple(want)
+        h = self.to_tuple(have)
+        if set(w).issubset(set(h)):
+            return None
+        else:
+            return want
+
+    @property
+    def state(self):
+        if self.want.state == 'disabled' and self.have.enabled:
+            return self.want.state
+        elif self.want.state in ['present', 'enabled'] and self.have.disabled:
+            return self.want.state
+
+    @property
+    def pools(self):
+        result = self._diff_complex_items(self.want.pools, self.have.pools)
+        return result
+
 
 class ModuleManager(object):
-    def __init__(self, client):
-        self.client = client
+    def __init__(self, *args, **kwargs):
+        self.module = kwargs.get('module', None)
+        self.client = kwargs.get('client', None)
+        self.kwargs = kwargs
 
     def exec_module(self):
         if self.version_is_less_than_12():
@@ -266,9 +391,9 @@ class ModuleManager(object):
 
     def get_manager(self, type):
         if type == 'typed':
-            return TypedManager(self.client)
+            return TypedManager(**self.kwargs)
         elif type == 'untyped':
-            return UntypedManager(self.client)
+            return UntypedManager(**self.kwargs)
 
     def version_is_less_than_12(self):
         version = self.client.api.tmos_version
@@ -279,11 +404,12 @@ class ModuleManager(object):
 
 
 class BaseManager(object):
-    def __init__(self, client):
-        self.client = client
-        self.have = None
-        self.want = Parameters(self.client.module.params)
-        self.changes = Parameters()
+    def __init__(self, *args, **kwargs):
+        self.module = kwargs.get('module', None)
+        self.client = kwargs.get('client', None)
+        self.want = ModuleParameters(params=self.module.params)
+        self.have = ApiParameters()
+        self.changes = UsableChanges()
 
     def _set_changed_options(self):
         changed = {}
@@ -291,24 +417,23 @@ class BaseManager(object):
             if getattr(self.want, key) is not None:
                 changed[key] = getattr(self.want, key)
         if changed:
-            self.changes = Parameters(changed)
+            self.changes = UsableChanges(params=changed)
 
     def _update_changed_options(self):
-        changed = {}
-        for key in Parameters.updatables:
-            if getattr(self.want, key) is not None:
-                attr1 = getattr(self.want, key)
-                attr2 = getattr(self.have, key)
-                if attr1 != attr2:
-                    changed[key] = attr1
-
-        if self.want.state == 'disabled' and self.have.enabled:
-            changed['state'] = self.want.state
-        elif self.want.state in ['present', 'enabled'] and self.have.disabled:
-            changed['state'] = self.want.state
-
+        diff = Difference(self.want, self.have)
+        updatables = Parameters.updatables
+        changed = dict()
+        for k in updatables:
+            change = diff.compare(k)
+            if change is None:
+                continue
+            else:
+                if isinstance(change, dict):
+                    changed.update(change)
+                else:
+                    changed[k] = change
         if changed:
-            self.changes = Parameters(changed)
+            self.changes = UsableChanges(params=changed)
             return True
         return False
 
@@ -325,20 +450,17 @@ class BaseManager(object):
         except iControlUnexpectedHTTPError as e:
             raise F5ModuleError(str(e))
 
-        changes = self.changes.to_return()
+        reportable = ReportableChanges(params=self.changes.to_return())
+        changes = reportable.to_return()
         result.update(**changes)
         result.update(dict(changed=changed))
-        self._announce_deprecations()
+        self._announce_deprecations(result)
         return result
 
-    def _announce_deprecations(self):
-        warnings = []
-        if self.want:
-            warnings += self.want._values.get('__warnings', [])
-        if self.have:
-            warnings += self.have._values.get('__warnings', [])
+    def _announce_deprecations(self, result):
+        warnings = result.pop('__warnings', [])
         for warning in warnings:
-            self.client.module.deprecate(
+            self.module.deprecate(
                 msg=warning['msg'],
                 version=warning['version']
             )
@@ -355,7 +477,7 @@ class BaseManager(object):
 
     def create(self):
         self._set_changed_options()
-        if self.client.check_mode:
+        if self.module.check_mode:
             return True
         self.create_on_device()
         return True
@@ -370,7 +492,7 @@ class BaseManager(object):
         self.have = self.read_current_from_device()
         if not self.should_update():
             return False
-        if self.client.check_mode:
+        if self.module.check_mode:
             return True
         self.update_on_device()
         return True
@@ -381,7 +503,7 @@ class BaseManager(object):
         return False
 
     def remove(self):
-        if self.client.check_mode:
+        if self.module.check_mode:
             return True
         self.remove_from_device()
         if self.exists():
@@ -410,7 +532,7 @@ class UntypedManager(BaseManager):
             partition=self.want.partition
         )
         result = resource.attrs
-        return Parameters(result)
+        return ApiParameters(params=result)
 
     def create_on_device(self):
         params = self.want.api_params()
@@ -430,17 +552,26 @@ class UntypedManager(BaseManager):
 
 
 class TypedManager(BaseManager):
-    def __init__(self, client):
-        super(TypedManager, self).__init__(client)
+    def __init__(self, *args, **kwargs):
+        super(TypedManager, self).__init__(**kwargs)
         if self.want.type is None:
             raise F5ModuleError(
                 "The 'type' option is required for BIG-IP instances "
                 "greater than or equal to 12.x"
             )
+        type_map = dict(
+            a='a_s',
+            aaaa='aaaas',
+            cname='cnames',
+            mx='mxs',
+            naptr='naptrs',
+            srv='srvs'
+        )
+        self.collection = type_map[self.want.type]
 
     def exists(self):
         wideips = self.client.api.tm.gtm.wideips
-        collection = getattr(wideips, self.want.collection)
+        collection = getattr(wideips, self.collection)
         resource = getattr(collection, self.want.type)
         result = resource.exists(
             name=self.want.name,
@@ -451,7 +582,7 @@ class TypedManager(BaseManager):
     def update_on_device(self):
         params = self.want.api_params()
         wideips = self.client.api.tm.gtm.wideips
-        collection = getattr(wideips, self.want.collection)
+        collection = getattr(wideips, self.collection)
         resource = getattr(collection, self.want.type)
         result = resource.load(
             name=self.want.name,
@@ -461,19 +592,19 @@ class TypedManager(BaseManager):
 
     def read_current_from_device(self):
         wideips = self.client.api.tm.gtm.wideips
-        collection = getattr(wideips, self.want.collection)
+        collection = getattr(wideips, self.collection)
         resource = getattr(collection, self.want.type)
         result = resource.load(
             name=self.want.name,
             partition=self.want.partition
         )
         result = result.attrs
-        return Parameters(result)
+        return ApiParameters(params=result)
 
     def create_on_device(self):
         params = self.want.api_params()
         wideips = self.client.api.tm.gtm.wideips
-        collection = getattr(wideips, self.want.collection)
+        collection = getattr(wideips, self.collection)
         resource = getattr(collection, self.want.type)
         resource.create(
             name=self.want.name,
@@ -483,7 +614,7 @@ class TypedManager(BaseManager):
 
     def remove_from_device(self):
         wideips = self.client.api.tm.gtm.wideips
-        collection = getattr(wideips, self.want.collection)
+        collection = getattr(wideips, self.collection)
         resource = getattr(collection, self.want.type)
         result = resource.load(
             name=self.want.name,
@@ -506,50 +637,60 @@ class ArgumentSpec(object):
         ]
         lb_method_choices = deprecated + supported
         self.supports_check_mode = True
-        self.argument_spec = dict(
-            lb_method=dict(
-                required=False,
+        argument_spec = dict(
+            pool_lb_method=dict(
                 choices=lb_method_choices,
-                default=None
+                aliases=['lb_method']
             ),
             name=dict(
                 required=True,
                 aliases=['wide_ip']
             ),
             type=dict(
-                required=False,
-                default=None,
                 choices=[
                     'a', 'aaaa', 'cname', 'mx', 'naptr', 'srv'
                 ]
             ),
             state=dict(
-                required=False,
                 default='present',
                 choices=['absent', 'present', 'enabled', 'disabled']
+            ),
+            pools=dict(
+                type='list',
+                options=dict(
+                    name=dict(required=True),
+                    ratio=dict(type='int')
+                )
+            ),
+            partition=dict(
+                default='Common',
+                fallback=(env_fallback, ['F5_PARTITION'])
             )
         )
-        self.f5_product_name = 'bigip'
+        self.argument_spec = {}
+        self.argument_spec.update(f5_argument_spec)
+        self.argument_spec.update(argument_spec)
 
 
 def main():
-    if not HAS_F5SDK:
-        raise F5ModuleError("The python f5-sdk module is required")
-
     spec = ArgumentSpec()
 
-    client = AnsibleF5Client(
+    module = AnsibleModule(
         argument_spec=spec.argument_spec,
-        supports_check_mode=spec.supports_check_mode,
-        f5_product_name=spec.f5_product_name
+        supports_check_mode=spec.supports_check_mode
     )
+    if not HAS_F5SDK:
+        module.fail_json(msg="The python f5-sdk module is required")
 
     try:
-        mm = ModuleManager(client)
+        client = F5Client(**module.params)
+        mm = ModuleManager(module=module, client=client)
         results = mm.exec_module()
-        client.module.exit_json(**results)
+        cleanup_tokens(client)
+        module.exit_json(**results)
     except F5ModuleError as e:
-        client.module.fail_json(msg=str(e))
+        cleanup_tokens(client)
+        module.fail_json(msg=str(e))
 
 
 if __name__ == '__main__':
