@@ -44,9 +44,10 @@ options:
         device config parser.
     required: false
     default: null
+    aliases: ['commands']
   parents:
     description:
-      - The ordered set of parents that uniquely identify the section
+      - The ordered set of parents that uniquely identify the section or hierarchy
         the commands should be checked against.  If the parents argument
         is omitted, the commands are checked against the set of top
         level or global commands.
@@ -63,6 +64,18 @@ options:
     required: false
     default: null
     version_added: "2.2"
+  replace_src:
+    description:
+      - The I(replace_src) argument provides path to the configuration file
+        to load into the remote system. This argument is used to replace the
+        entire config with a flat-file. This is used with argument I(replace)
+        with value I(config). This is mutually exclusive with the I(lines) and
+        I(src) arguments. This argument is supported on Nexus 9K device.
+        Use I(nxos_file_copy) module to copy the flat file to remote device and
+        then use the path with this argument.
+    required: false
+    default: null
+    version_added: "2.5"
   before:
     description:
       - The ordered set of commands to push on to the command stack if
@@ -100,10 +113,10 @@ options:
         the modified lines are pushed to the device in configuration
         mode.  If the replace argument is set to I(block) then the entire
         command block is pushed to the device in configuration mode if any
-        line is not correct.
+        line is not correct. I(replace config) is supported only on Nexus 9K device.
     required: false
-    default: lineo
-    choices: ['line', 'block']
+    default: line
+    choices: ['line', 'block', 'config']
   force:
     description:
       - The force argument instructs the module to not consider the
@@ -249,6 +262,12 @@ EXAMPLES = """
     parents: ip access-list test
     before: no ip access-list test
     replace: block
+
+- name: replace config with flat file
+  nxos_config:
+    replace_src: config.txt
+    replace: config
+
 """
 
 RETURN = """
@@ -268,12 +287,16 @@ backup_path:
   type: string
   sample: /playbooks/ansible/backup/nxos_config.2016-07-16@22:28:34
 """
+
+
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.netcfg import NetworkConfig, dumps
-from ansible.module_utils.nxos import get_config, load_config, run_commands
-from ansible.module_utils.nxos import nxos_argument_spec
-from ansible.module_utils.nxos import check_args as nxos_check_args
-from ansible.module_utils.network_common import to_list
+from ansible.module_utils.connection import ConnectionError
+from ansible.module_utils.network.common.config import NetworkConfig, dumps
+from ansible.module_utils.network.nxos.nxos import get_config, load_config, run_commands
+from ansible.module_utils.network.nxos.nxos import get_capabilities
+from ansible.module_utils.network.nxos.nxos import nxos_argument_spec
+from ansible.module_utils.network.nxos.nxos import check_args as nxos_check_args
+from ansible.module_utils.network.common.utils import to_list
 
 
 def get_running_config(module, config=None):
@@ -290,7 +313,10 @@ def get_running_config(module, config=None):
 def get_candidate(module):
     candidate = NetworkConfig(indent=2)
     if module.params['src']:
-        candidate.load(module.params['src'])
+        if module.params['replace'] != 'config':
+            candidate.load(module.params['src'])
+    if module.params['replace'] == 'config':
+        candidate.load('config replace {0}'.format(module.params['replace_src']))
     elif module.params['lines']:
         parents = module.params['parents'] or list()
         candidate.add(module.params['lines'], parents=parents)
@@ -300,9 +326,9 @@ def get_candidate(module):
 def execute_show_commands(module, commands, output='text'):
     cmds = []
     for command in to_list(commands):
-        cmd = { 'command': command,
-                'output': output,
-              }
+        cmd = {'command': command,
+               'output': output,
+               }
         cmds.append(cmd)
     body = run_commands(module, cmds)
     return body
@@ -313,7 +339,7 @@ def main():
     """
     argument_spec = dict(
         src=dict(type='path'),
-
+        replace_src=dict(),
         lines=dict(aliases=['commands'], type='list'),
         parents=dict(type='list'),
 
@@ -321,7 +347,7 @@ def main():
         after=dict(type='list'),
 
         match=dict(default='line', choices=['line', 'strict', 'exact', 'none']),
-        replace=dict(default='line', choices=['line', 'block']),
+        replace=dict(default='line', choices=['line', 'block', 'config']),
 
         running_config=dict(aliases=['config']),
         intended_config=dict(),
@@ -343,12 +369,14 @@ def main():
 
     argument_spec.update(nxos_argument_spec)
 
-    mutually_exclusive = [('lines', 'src'),
+    mutually_exclusive = [('lines', 'src', 'replace_src'),
+                          ('parents', 'src'),
                           ('save', 'save_when')]
 
     required_if = [('match', 'strict', ['lines']),
                    ('match', 'exact', ['lines']),
                    ('replace', 'block', ['lines']),
+                   ('replace', 'config', ['replace_src']),
                    ('diff_against', 'intended', ['intended_config'])]
 
     module = AnsibleModule(argument_spec=argument_spec,
@@ -363,19 +391,36 @@ def main():
 
     config = None
 
+    try:
+        info = get_capabilities(module)
+        api = info.get('network_api', 'nxapi')
+        device_info = info.get('device_info', {})
+        os_platform = device_info.get('network_os_platform', '')
+    except ConnectionError:
+        api = ''
+        os_platform = ''
+
+    if api == 'cliconf' and module.params['replace'] == 'config':
+        if '9K' not in os_platform:
+            module.fail_json(msg='replace: config is supported only on Nexus 9K series switches')
+
+    if module.params['replace_src']:
+        if module.params['replace'] != 'config':
+            module.fail_json(msg='replace: config is required with replace_src')
+
     if module.params['backup'] or (module._diff and module.params['diff_against'] == 'running'):
         contents = get_config(module)
         config = NetworkConfig(indent=2, contents=contents)
         if module.params['backup']:
             result['__backup__'] = contents
 
-    if any((module.params['src'], module.params['lines'])):
+    if any((module.params['src'], module.params['lines'], module.params['replace_src'])):
         match = module.params['match']
         replace = module.params['replace']
 
         candidate = get_candidate(module)
 
-        if match != 'none':
+        if match != 'none' and replace != 'config':
             config = get_running_config(module, config)
             path = module.params['parents']
             configobjs = candidate.difference(config, match=match, replace=replace, path=path)
@@ -455,12 +500,18 @@ def main():
             base_config = NetworkConfig(indent=1, contents=contents, ignore_lines=diff_ignore_lines)
 
             if running_config.sha1 != base_config.sha1:
+                if module.params['diff_against'] == 'intended':
+                    before = running_config
+                    after = base_config
+                elif module.params['diff_against'] in ('startup', 'running'):
+                    before = base_config
+                    after = running_config
+
                 result.update({
                     'changed': True,
-                    'diff': {'before': str(base_config), 'after': str(running_config)}
+                    'diff': {'before': str(before), 'after': str(after)}
                 })
 
-
     module.exit_json(**result)
 
 
